<?php

namespace Drupal\dbb\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\node\Controller\NodeViewController;
use pimax\FbBotApp;
use pimax\Messages\Message;
use pimax\Messages\MessageButton;
use pimax\Messages\QuickReplyButton;
use pimax\Messages\StructuredMessage;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;

/**
 * Custom node redirect controller.
 */
class NodeRedirectController extends NodeViewController
{
  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  private $requestStack;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Route Match to get Node from.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * The paragraph storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $phStorage;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container)
  {
    $instance = parent::create($container);
    $instance->requestStack = $container->get('request_stack');
    $instance->logger = $container->get('logger.factory')->get('dbb');
    $instance->currentRouteMatch = $container->get('current_route_match');
    $instance->phStorage = $container->get('entity_type.manager')->getStorage('paragraph');
    return $instance;
  }

  /**
   * Veiw node cnonical handler.
   *
   * @param \Drupal\Core\Entity\EntityInterface $node
   *   Node content type.
   * @param string $view_mode
   *   View mode passed to route handler.
   * @param string $langcode
   *   Language code passed to route handler.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Return void.
   */
  public function view(EntityInterface $node, $view_mode = 'full', $langcode = null)
  {
    // Redirect to the edit path on the chatbot_webhook content type.
    if ($node->getType() === 'chatbot_webhook') {
      $request = $this->requestStack->getCurrentRequest();
      // Log each get request if hub_mode parameter is not empty.
      if (!empty($request->query->get('hub_mode')) &&
        $hub_verify_token = $request->query->get('hub_verify_token')) {
        $this->logger->notice('Request from facebook, hub_mode: @hub_mode, hub_verify_token=> @hvt, hub_challenge=> @hc', [
          '@hub_mode' => $request->query->get('hub_mode'),
          '@hvt' => $hub_verify_token,
        ]);
        // 'hub_verify_token';
        if ($hub_verify_token === $node->field_fb_verify_token->value) {
          echo($request->query->get('hub_challenge'));
          return new Response('', 200);
        }
      }
      if ($input = json_decode(file_get_contents('php://input'), true)) {
        // Handle delivery notifications.
        if (!empty($input['entry'][0]['messaging'][0]['delivery'])) {
          $this->logger->notice('Delivery confirmation, post_input=> @input', [
            '@input' => json_encode($input),
          ]);
          return new Response('', 200);
        }
        // Handle read notifications.
        if (!empty($input['entry'][0]['messaging'][0]['read'])) {
          $this->logger->notice('Read confirmation, post_input=> @input', [
            '@input' => json_encode($input),
          ]);
          return new Response('', 200);
        }
        // Handle message echo.
        if (isset($input['entry'][0]['messaging'][0]['message']['is_echo']) &&
          $input['entry'][0]['messaging'][0]['message']['is_echo']) {
          $this->logger->notice('TODO: Echo message, post_input=> @input', [
            '@input' => json_encode($input),
          ]);
          return new Response('', 200);
        }
        // Handle undefined Facebook request to webhook.
        if (!isset($input['entry'][0]['messaging'])) {
          $this->logger->notice('TODO: Handle undefined fb request, post_input=> @input', [
            '@input' => json_encode($input),
          ]);
          return new Response('', 200);
        }

        // Handle messaging.
        $this->logger->notice('Request from facebook, post_input=> @input', [
          '@input' => json_encode($input),
        ]);

        $sender = $input['entry'][0]['messaging'][0]['sender']['id'];
        if (isset($input['entry'][0]['messaging'][0]['message'])) {
          $message = $input['entry'][0]['messaging'][0]['message']['text'];
        } elseif (isset($input['entry'][0]['messaging'][0]['postback'])) {
          $payload = $input['entry'][0]['messaging'][0]['postback']['payload'];
          $message = 'payload-> ' . $payload . ' | ' . $input['entry'][0]['messaging'][0]['postback']['title'];
        }

        $this->sender = $sender;
        $this->token = $node->field_fb_page_access_token->value;

        $this->logger->notice('Request from facebook, sender=>@sender, message=> @msg', [
          '@sender' => $sender,
          '@msg' => $message,
        ]);
        // Handle payload postback_welcome it is payload for new visitors whom press GetStarted button.
        if ($payload == 'postback_welcome') {
          // Do nothing for now, cause we have implemented default mesasge in facebook to be sent to user
          $this->logger->notice('"postback_welcome" implement handler later.', [
            '@input' => json_encode($input),
          ]);
          return new Response('', 200);
        }
        // handle start button from old menu
        if ($payload == 'start') {
          // Do nothing for now, cause we have implemented default mesasge in facebook to be sent to user
          $this->logger->notice('"start" implement handler later.', [
            '@input' => json_encode($input),
          ]);
          return new Response('', 200);
        }
        // Handle payload from paragraph's id.
        if ($payload) {
          $this->handlePayload($payload);
          return new Response('', 200);
        }
        //TODO: Add handle multiple field_step values
        $step_payload = $node->field_fb_app_webhook->first()->target_id;
        // TODO: Check if this is actual rendered data for paragraph or abandoned.
        $this->handlePayload($step_payload);
        return new Response('', 200);
      }
      // Prevent visibility if not author.
      return new RedirectResponse('/node/' . $node->id() . '/edit');
    }
    // Otherwise, fall back to the parent route controller.
    else {
      return parent::view($node, $view_mode, $langcode);
    }
  }

  /**
   * Handler for payload - Step paragraph target_id.
   *
   * @param string|int $pl
   *   Payload of step paragraph.
   * @return void
   *   Result of handler performing.
   */
  private function handlePayload($pl)
  {
    $step = $this->phStorage->load($pl);
    $this->logger->notice('$step is: <br /> @print_r', [
      '@print_r' => '<pre>' . print_r($step, true) . '</pre>',
    ]);
    $this->logger->notice('$step->field_step->first()->entity->getType()=> @type', [
      '@type' => $step->field_step->first()->entity->getType(),
    ]);
    // Added handle multiple field_step values
    $steps = $step->get('field_step');
    foreach ($steps->referencedEntities() as $delta => $step_entity) {
      $this->logger->notice('> @count $step->field_step->first()->entity->getType()=> @type', [
        '@type' => $step->field_step->first()->entity->getType(),
        '@count' => $step->field_step->count()
      ]);

      switch ($step_entity->getType()) {
        case 'text':
          $message_to_reply = $step_entity->field_text->value;
          $this->sendText($message_to_reply);
          continue 2;
        case 'structured_message':
          $flow_id = $step_entity->field_flow->first()->target_id;
          $this->handleFlow($flow_id);
          continue 2;
        default:
          # code...
          continue 2;
      }
    }
    // switch ($step->field_step->first()->entity->getType()) {
    //   case 'text':
    //     $message_to_reply = $step->field_step->first()->entity->field_text->value;
    //     $this->sendText($message_to_reply);
    //     break;
    //   case 'structured_message':
    //     $flow_id = $step->field_step->first()->entity->field_flow->first()->target_id;
    //     $this->handleFlow($flow_id);
    //     break;
    //   default:
    //     # code...
    //     break;
    // }
    return;
  }

  /**
   * FB messenger API Send answer for structured message.
   *
   * @param [type] $flow_id
   * @return void
   */
  private function handleFlow($flow_id)
  {
    $flow = $this->phStorage->load($flow_id);
    $this->logger->notice('Flow type=> @flow', [
      '@flow' => $flow->getType(),
    ]);
    switch ($flow->getType()) {
      case 'message_buttons':
        $btn_text = $flow->field_msg_btn_text->value;
        // $step->field_step->first()->entity->getType()
        $buttons = $flow->field_msg_btn_buttons->getValue();
        // [NULL-35] Implement pimax library inside sendButtons method
        $this->sendButtons($btn_text, $buttons);
        break;

      case 'message_generic':
        // $this->handleGenericMessage("With Images, Titles, Subtitles and few Buttons on each card");
        break;

      default:
        # code...
        break;
    }
    return;
  }

  /**
   * FB messenger API Send Button answer.
   *
   * @param [type]|null $btn_text
   *   Text above buttons.
   * @paran array $buttons
   *   Array of referenced buttons target_id
   * @return void
   */
  private function sendButtons($btn_text, $buttons)
  {
    // HOWTO send answer with pimax library?
    // - Make bot instance
    // - make sure use statement are exists.
    // @see https://github.com/pimax/fb-messenger-php-example/blob/master/index.php
    $bot = new FbBotApp($this->token);
    // $bot->send(new Message($this->sender, $btn_text));
    $this->logger->notice('Buttons=> @btype', [
      '@btype' => json_encode($buttons),
    ]);
    $buttons_array = [];
    foreach ($buttons as $button) {
      $this->logger->notice('Each Button=> @btype', [
        '@btype' => json_encode($this->phStorage->load($button['target_id'])->field_btn_exec->entity->getType()),
      ]);
      switch ($this->phStorage->load($button['target_id'])->field_btn_exec->entity->getType()) {
        case 'button_link':
          $this->logger->notice('Link Button=> @blink', [
            '@blink' => json_encode($this->phStorage->load($button['target_id'])->field_btn_exec->entity->field_url->uri),
          ]);
          $buttons_array[] = new MessageButton(
            MessageButton::TYPE_WEB,
            $this->phStorage->load($button['target_id'])->field_btn_caption->value,
            $this->phStorage->load($button['target_id'])->field_btn_exec->entity->field_url->uri
          );
          break;

        default:
          $buttons_array[] = new MessageButton(
            MessageButton::TYPE_POSTBACK,
            $this->phStorage->load($button['target_id'])->field_btn_caption->value,
            $this->phStorage->load($button['target_id'])->field_btn_exec->target_id
          );
          break;
      }
    }

    // $buttons_array[] = new MessageButton(MessageButton::TYPE_WEB, 'Web link', 'http://www.alefinvest.xyz');

    $bot->send(new StructuredMessage(
      $this->sender,
      StructuredMessage::TYPE_BUTTON,
      [
        'text' => $btn_text,
        'buttons' => $buttons_array,
      ],
      [
        new QuickReplyButton(QuickReplyButton::TYPE_TEXT, 'QR button', 'get_qr_image'),
      ]
    ));
  }

  /**
   * FB messenger API Send text answer.
   *
   * @param string $message_to_reply
   * @return void
   */
  private function sendText($message_to_reply)
  {
    $sender = $this->sender;
    $access_token = $this->token;
    $this->logger->notice('message_to_reply=> @msg', [
      '@msg' => $message_to_reply,
    ]);
    // API Url which should be in backend form instead of adding in code
    $url = 'https://graph.facebook.com/v2.6/me/messages?access_token=' . $access_token;
    // Initiate cURL.
    $ch = curl_init($url);
    // The JSON data.
    $jsonData = '{
      "recipient":{
        "id":"' . $sender . '"
      },
      "message":{
        "text":"' . $message_to_reply . '"
      }
    }';
    // Encode the array into JSON.
    $jsonDataEncoded = $jsonData;
    // Tell cURL that we want to send a POST request.
    curl_setopt($ch, CURLOPT_POST, 1);
    // Attach our encoded JSON string to the POST fields.
    curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonDataEncoded);
    // Set the content type to application/json
    curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
    // curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
    // Execute the request.
    $result = curl_exec($ch);
    return $result;
  }
}
